@@ -265,7 +265,7 @@ class IDServerApp(PApplication):
                 ctrl.enabled = subctrl.enabled = False
             for ctrl in self.other_controls:
                 ctrl.enabled = False
-            self.button_go.set_caption(CAPTION_STOP)
+            self.button_go.caption = CAPTION_STOP
             self.button_go.set_icon(ICON_STOP)
             
             self.client = IDServerUIClient(self.transport, self.outputfile, self)
@@ -276,7 +276,7 @@ class IDServerApp(PApplication):
     def done(self):
         # Either we're done, or we've stopped a query in progress
         self.button_save.enabled = True
-        self.button_go.set_caption(CAPTION_GO)
+        self.button_go.caption = CAPTION_GO
         self.button_go.set_icon(ICON_GO)
         for ctrl in self.other_controls:
             ctrl.enabled = True
