@@ -17,15 +17,120 @@ import os
 
 from plib.stdlib.builtins import first
 
+
+# Must do this before any other plib.ui imports or action
+# (Note that we don't set a specific toolkit here, but apps
+# that want to do that must do it in this section)
+
+from plib.ui import custom
+
+custom.add_signals(
+    SIGNAL_CUSTOM1=(('SIGNAL_FINISHED', 50), 'custom1'),
+    SIGNAL_CUSTOM2=(('SIGNAL_FINISHED', 51), 'custom2'),
+    SIGNAL_EMITTED=(('SIGNAL_FINISHED', 60), 'emitted'),
+)
+
+custom.add_widget_prefixes(
+    ClickableLabel='label',
+    CustomWidget='widget',
+)
+
+
 from plib.ui import __version__
 from plib.ui.defs import *
 from plib.ui.app import PApplication
 from plib.ui.dialogs import StringSelectDialog
+from plib.ui.imp import get_toolkit_object
 from plib.ui.manager import PWidgetManager
 from plib.ui.output import PTextOutput
+from plib.ui.toolkits import get_toolkit
 from plib.ui.widgets import *
 
 
+# TODO: is there a way to add an API to custom to avoid toolkit-specific code here?
+
+class MouseBase:
+    
+    mouse_pressed = False
+    
+    @property
+    def event_signal(self):
+        return self.signals[0]
+
+
+toolkit = get_toolkit()
+
+if toolkit in ('pyside2', 'qt5'):
+    
+    class MouseMixin(MouseBase):
+        
+        def mousePressEvent(self, event):
+            self.mouse_pressed = True
+        
+        def mouseReleaseEvent(self, event):
+            if self.mouse_pressed:
+                self.do_notify(self.event_signal)
+            self.mouse_pressed = False
+
+
+elif toolkit == 'wx':
+    
+    class MouseMixin(MouseBase):
+        
+        def setup_signals(self):
+            import wx
+            self.wx_register_event(wx.EVT_LEFT_DOWN, self.OnLeftDown)
+            self.wx_register_event(wx.EVT_LEFT_UP, self.OnLeftUp)
+        
+        def OnLeftDown(self, event):
+            self.mouse_pressed = True
+        
+        def OnLeftUp(self, event):
+            if self.mouse_pressed:
+                self.forward_event(self.event_signal, event)
+            self.mouse_pressed = False
+
+
+else:
+    # This should never happen
+    raise RuntimeError("Unknown UI toolkit {}".format(toolkit))
+
+
+PTextLabel = get_toolkit_object('label', 'PTextLabel')
+
+
+class ClickableLabel(MouseMixin, PTextLabel):
+    
+    signals = (
+        SIGNAL_CLICKED,
+    )
+
+
+class CustomWidget(MouseMixin, PTextLabel):
+    
+    signals = (
+        SIGNAL_EMITTED,
+    )
+
+
+def clickable_label(name, caption, geometry=None, font=None):
+    args = (caption,)
+    kwargs = dict(
+        geometry=geometry,
+        font=font,
+    )
+    return ('__main__', 'ClickableLabel', name, args, kwargs)
+
+
+def custom_widget(name, caption, geometry=None, font=None):
+    args = (caption,)
+    kwargs = dict(
+        geometry=geometry,
+        font=font,
+    )
+    return ('__main__', 'CustomWidget', name, args, kwargs)
+
+
 def display_sub_objects(mgr, mgr_name='app'):
     print("Showing sub-objects for {}".format(mgr_name))
     for widget in mgr.all_widgets:
@@ -46,7 +151,16 @@ class Page(PWidgetManager):
     name = 'manager'
     
     main_widget = tab(ALIGN_JUST, LAYOUT_VERTICAL, [
+        box(ALIGN_TOP, LAYOUT_VERTICAL, [
+            clickable_label('labeltest', "Clickable Label Test"),
+        ]),
         listbox('numbers', number_strings),
+        box(ALIGN_BOTTOM, LAYOUT_HORIZONTAL, [
+            button('custom1', "Custom Signal 1"),
+            button('custom2', "Custom Signal 2"),
+            padding(),
+            custom_widget('custom3', "Custom Widget"),
+        ]),
     ])
     
     def setup_signals(self):
@@ -77,6 +191,20 @@ class Page(PWidgetManager):
         self.app.output_message("From Page: SIGNAL_LISTBOXSELECTED {} {}".format(
             index, self.listbox_numbers.current_text()
         ))
+    
+    def on_labeltest_clicked(self):
+        self.app.output_message("From Page ClickableLabel: SIGNAL_CLICKED")
+    
+    def on_custom1_clicked(self):
+        print("custom1_clicked")
+        self.app.do_notify(SIGNAL_CUSTOM1)
+    
+    def on_custom2_clicked(self):
+        print("custom2_clicked")
+        self.app.do_notify(SIGNAL_CUSTOM2)
+    
+    def on_custom3_emitted(self):
+        self.app.output_message("From Page CustomWidget: SIGNAL_EMITTED")
 
 
 PAGE_TABS = 0
@@ -206,6 +334,11 @@ def panels(name, contents):
 
 class UISignalTester(PApplication):
     
+    signals = (
+        SIGNAL_CUSTOM1,
+        SIGNAL_CUSTOM2,
+    )
+    
     page_select = PAGE_TABS
     select_layout = None
     
@@ -425,6 +558,12 @@ class UISignalTester(PApplication):
             str(row), str(col), self.table_cells.current_cell())
         )
     
+    def on_app_custom1(self):
+        self.output_message("SIGNAL_CUSTOM1")
+    
+    def on_app_custom2(self):
+        self.output_message("SIGNAL_CUSTOM2")
+    
     queryclose_handled = False
     
     def on_app_queryclose(self):
