@@ -8,7 +8,7 @@ Released under the GNU General Public License, Version 2
 See the LICENSE and README files for more information
 """
 
-from plib.ui.widgets import get_toolkit_class
+from plib.ui.imp import get_toolkit_object
 
 
-PApplication = get_toolkit_class('app', 'PApplication')
+PApplication = get_toolkit_object('app', 'PApplication')
