@@ -18,7 +18,8 @@ from plib.stdlib.decotools import cached_property
 from plib.ui import __version__
 from plib.ui.defs import *
 from plib.ui.common import *
-from plib.ui.widgets import setup_signal_handlers, get_toolkit_class, widget_from_spec
+from plib.ui.imp import get_toolkit_object
+from plib.ui.widgets import setup_signal_handlers, widget_from_spec
 
 
 class PSignalBase(object):
@@ -643,9 +644,7 @@ class PApplicationBase(PControllerBase, PSignalBase):
     """Base class for GUI application.
     """
     
-    signals = (
-        # SIGNAL_BEFOREQUIT
-    )
+    signals = ()
     
     handler_prefix = 'app'
     
@@ -716,7 +715,7 @@ class PApplicationBase(PControllerBase, PSignalBase):
             ('mainwin', 'PMainWindow') if self.use_mainwindow else
             ('app', 'PTopWindow')
         )
-        main_klass = get_toolkit_class(*args)
+        main_klass = get_toolkit_object(*args)
         main_window = main_klass(self)
         main_window.set_caption(self.main_title)
         return main_window
