@@ -191,3 +191,59 @@ WIDTH_STRETCH = -1
 def add_constants(**kw):
     for name, value in kw.items():
         setattr(sys.modules[__name__], name, value)
+
+
+# Maps for widget construction and event handler connection
+
+WIDGET_EVENT_MAP = {
+    SIGNAL_ACTIVATED: None,
+    SIGNAL_CLICKED: 'clicked',
+    SIGNAL_TOGGLED: 'toggled',
+    SIGNAL_SELECTED: 'selected',
+    SIGNAL_BUTTONSELECTED: 'selected',
+    SIGNAL_LISTVIEWSELECTED: 'selected',
+    SIGNAL_LISTBOXSELECTED: 'selected',
+    SIGNAL_CELLSELECTED: 'selected',
+    SIGNAL_FOCUS_IN: 'focus_in',
+    SIGNAL_FOCUS_OUT: 'focus_out',
+    SIGNAL_CELLCHANGED: 'changed',
+    SIGNAL_TEXTCHANGED: 'changed',
+    SIGNAL_TEXTMODCHANGED: 'mod_changed',
+    SIGNAL_TEXTSTATECHANGED: 'state_changed',
+    SIGNAL_EDITCHANGED: 'changed',
+    SIGNAL_ENTER: 'enter',
+    SIGNAL_TABSELECTED: 'selected',
+    SIGNAL_QUERYCLOSE: 'queryclose',
+    SIGNAL_CLOSING: 'closing',
+    SIGNAL_SHOWN: 'shown',
+    SIGNAL_HIDDEN: 'hidden',
+}
+
+add_handlers = WIDGET_EVENT_MAP.update
+
+WIDGET_ATTR_PREFIXES = {
+    # Only widgets whose prefix is different from their module name need this
+    'PCheckBox': 'checkbox',
+    'PEditBox': 'edit',
+    'PNumEditBox': 'edit',
+    'PPanel': 'panel',
+    'PButtonGroup': 'buttongroup',
+    'PRadioGroup': 'radiogroup',
+    'PTreeView': 'treeview',
+    'PEditControl': 'memo',
+    'PTextDisplay': 'text',
+    'PStatusLabel': 'label',
+}
+
+add_widget_prefixes = WIDGET_ATTR_PREFIXES.update
+
+WIDGET_AUTO_PREFIXES = [
+    # These prefixes are used in auto_name and should not be further prefixed
+    'groupbox_',
+    'padding_',
+    'panel_',
+    'labelbox_',
+]
+
+add_auto_prefix = WIDGET_AUTO_PREFIXES.append
+add_auto_prefixes = WIDGET_AUTO_PREFIXES.extend
