@@ -218,6 +218,20 @@ event_map = {
     SIGNAL_BEFOREQUIT: wx.EVT_WINDOW_DESTROY
 }
 
+
+# This is provided to support the custom API; it must be called before
+# any widget classes are declared
+
+# Note: args are ignored for this toolkit (unlike pyside2, qt5)
+
+def add_custom_signal(signame, *args):
+    # Importing all from defs above will have retrieved the custom signal by name
+    sig = getattr(sys.modules[__name__], signame)
+    event, binder = NewEvent()
+    custom_map[sig] = event
+    event_map[sig] = binder
+
+
 # 'Wrapper' functions for certain events to repackage parameters
 
 def wx_plain_wrapper(self, target):
@@ -305,6 +319,7 @@ no_wrapper_events = set([
     SIGNAL_SHOWEVENT,
     SIGNAL_MINIMIZEEVENT,
     SIGNAL_CLOSEEVENT,
+    SIGNAL_FINISHED,  # this is a special case, the handler just takes a boolean
 ])
 
 
@@ -741,6 +756,14 @@ class PApplication(PWxSignal, wx.App, PApplicationBase):
         PApplicationBase.__init__(self, arglist)
         wx.App.__init__(self, arglist)
     
+    def setup_notify(self, signal, target):
+        # wx.App can't register events, so we fake them as coming from the main window
+        self.main_window.setup_notify(signal, target)
+    
+    def do_notify(self, signal, *args):
+        # wx.App can't send events, so we fake them as coming from the main window
+        self.main_window.do_notify(signal, *args)
+    
     def about_toolkit(self):
         # No built-in "About Wx" dialog
         self.message_box.info("About Wx", "WxPython {}".format(wx.version()))
